#####################################################################
# 
# Xron -- Zope Scheduled Event Product
# Copyright (C) 2000 Loren Stafford
#  
# Derived from the ZScheduler product version 0.0.7 in accord 
# with the terms of the ZScheduler's license.
#
# Based on code by Martijn Pieters, (c) 1999 Antraciet B.V.
# Used with explicit permission.
# 
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License  
# along with this program; if not, write to the 
#     Free Software Foundation, Inc., 
#     59 Temple Place - Suite 330, 
#     Boston, MA  02111-1307, USA.
# 
#####################################################################
""" Scheduled DTML Methods for Xron """

from Globals import HTMLFile, default__class_init__
try:
  from Globals import DateTime
except ImportError:
  from DateTime import DateTime
from OFS.DTMLMethod import DTMLMethod
from Products.ZCatalog.CatalogAwareness import CatalogAware
from urllib import quote
import string

import Schedule
ScheduleID=Schedule.ScheduleID

#import pdb; pdb.set_trace()

class BaseXronDTMLMethod(CatalogAware, DTMLMethod):
  meta_type = 'Xron DTML Method'
  default_catalog = ScheduleID

  def __init__(self, id, title = '', file = ''):
    DTMLMethod.__init__(self, file, __name__ = id)

  manage_options=(
      {'label':'Edit', 'action':'manage_main', 
       'help':('Xron','XronDTMLMethod_Edit.stx')},
      {'label':'Upload', 'action':'manage_uploadForm'},
      {'label':'View', 'action':''},
      {'label':'Trigger', 'action':'manage_triggerXronEvent'},
      {'label':'Proxy', 'action':'manage_proxyForm'},
      {'label':'Security', 'action':'manage_access'},
                 )
  __ac_permissions__=(
    ('View management screens', (
        'manage_editForm', 'manage', 'manage_main', 'manage_uploadForm',
        'document_src', 'PrincipiaSearchSource')),
    ('Change DTML Methods', ('manage_edit', 'manage_upload', 'PUT')),
    ('Change proxy roles', ('manage_proxyForm', 'manage_proxy')),
    ('View', ('__call__', '')),
    ('FTP access', ('manage_FTPstat','manage_FTPget','manage_FTPlist')),
  )

  manage_editForm=HTMLFile('dtml/documentEdit', globals())
  manage=manage_main=manage_editDocument=manage_editForm
  try:
    manage_editForm._setName("manage_editForm")
  except AttributeError:
    pass
  import sys, string

  def nextEventTime():
    raise NotImplemented
      
  # Prevent changing the default_catalog
  manage_editCatalogerForm = None
  def manage_editCataloger(self, default):
    raise NotImplemented

  # Only index if nextEventTime returns something
  def index_object(self):
    if self.nextEventTime() is not None:
        CatalogAware.index_object.im_func(self)
        # see Python Reference Manual "The standard type hierarchy" 
        # for the built-in type im_func

  def disarm(self):
    """Private method for use only by the Dispatcher,
    when trigger fails.
    Marks the event as fired by nullifying its time.
    """
    self.executeAt=DateTime(0)
    self.reindex_object()

  def _arm(self, eventTime):
    """Private method for use only by the trigger to complete rescheduling
    Hides the rescheduling mechanism from the rescheduling
    policy implemented by overriddable method self.reschedule();
    Records the event in the Schedule catalog.
    """
    self.executeAt=eventTime
    self.reindex_object()

  def reschedule(self,REQUEST=None):
    """Implement rescheduling policy.
    This method is designed to be overridden by a ZClass DTML-method;
    so, it passes parameters via REQUEST.
    input:  REQUEST.XronEventOldTime a DateTime object
    output: REQUEST.XronEventTime a DateTime object
    Calculate REQUEST.XronEventTime as a function of REQUEST.XronEventOldTime
    """
    if self.periodDays or self.frequency=='end of the month':
      if not (self.frequency in ['years','months','end of the month']):
        if self.frequency=='seconds':
          periodInDays=float(self.periodDays)/86400.0
        elif self.frequency=='minutes':
          periodInDays=float(self.periodDays)/1440.0
        elif self.frequency=='hours':
          periodInDays=float(self.periodDays)/24.0
        elif self.frequency=='weeks':
          periodInDays=float(self.periodDays)*7.0
        else:
          periodInDays=float(self.periodDays)
        REQUEST.set('XronEventTime',REQUEST.XronEventOldTime+periodInDays)
      else:
        nextDay=str(self.originalDate.day())
        nextHour=str(REQUEST.XronEventOldTime.h_24())
        nextMinute=str(REQUEST.XronEventOldTime.minute())
        nextMonth=int(REQUEST.XronEventOldTime.mm())
        nextYear=str(REQUEST.XronEventOldTime.year())
        if self.frequency=='years':
          nextYear=str(int(nextYear)+self.periodDays)
        elif self.frequency=='months':
          futureYears=int(self.periodDays/12)
          futureMonths=self.periodDays%12
          nextMonth=nextMonth+futureMonths
          nextYear=str(int(nextYear)+futureYears)
        elif self.frequency=='end of the month':
          nextMonth=nextMonth+1
          nextDay='31'
        if nextMonth>=13:
          nextMonth=nextMonth-12
          nextYear=str(int(nextYear)+1)
        intNextDay=int(nextDay)
        if (int(nextYear)%4==0) and (nextMonth==2) and (intNextDay>29):
          nextDay='29'
        elif (int(nextYear)%4!=0) and (nextMonth==2) and (intNextDay>28):
          nextDay='28'
        elif (nextMonth in [4,6,9,11]) and (intNextDay>30):
          nextDay='30'
        nextDate=DateTime(nextYear+'/'+str(nextMonth)+'/'+nextDay+' '+nextHour+':'+nextMinute)
        REQUEST.set('XronEventTime', nextDate)
    else:
        REQUEST.set('XronEventTime', DateTime(0))

  def trigger(self, client=None, REQUEST=None, RESPONSE=None, **kw):
    """For use by the Dispatcher to fire the event.
    Use also from the browser URL box to debug your events.
    Reschedules the event, then calls the action method.
    """
    if REQUEST is not None:
      REQUEST.set('XronEventOldTime',self.executeAt)
      self.reschedule(REQUEST=REQUEST)
      XronEventTime=REQUEST.XronEventTime
      #self._arm(XronEventTime)
      self.arm(REQUEST=REQUEST)
    else:
      self.disarm()
    #import pdb; pdb.set_trace() # debug
    try:
      return self.__call__(client=self, 
        REQUEST=REQUEST, RESPONSE=RESPONSE, kw=kw)
    except:
      import sys, traceback, string
      type, val, tb = sys.exc_info()
      fe=traceback.format_exception (type, val, tb)
      return string.join(fe, '\n')
      
  def manage_triggerXronEvent(self, client=None, REQUEST=None, RESPONSE=None, **kw):
    """For use by the Trigger tab on XronDTMLMethod management page.
    Use for testing the XronDTMLMethod in a request context.
    """
    aurl=REQUEST['URL1']
    furl=string.join((aurl, 'trigger'), '/')
    RESPONSE.redirect(furl)

  def arm(self, REQUEST=None):
    """Method to aid in programmatically setting 
    executeAt from a DTMLMethod.
    input: REQUEST.XronEventTime a DateTime object
    """
    self.executeAt=REQUEST.XronEventTime
    self.reindex_object()

default_event_html = 'Content-type: text/plain\n\n'

class XronDTMLMethod(BaseXronDTMLMethod):
  #meta_type = 'Xron DTML Method'

  def __init__(self,id,executeAt,periodDays,frequency,title='',file=''):
    BaseXronDTMLMethod.__init__(self,id,title,file)
    self.executeAt=executeAt
    self.originalDate=executeAt
    self.periodDays=periodDays
    self.frequency=frequency
    self.title=title

  def nextEventTime(self):
    #raise DEBUG, self.executeAt
    #if DateTime().lessThanEqualTo(self.executeAt):
    #    return self.executeAt
    #return None
    return self.executeAt

  def manage_edit(self,data,title,submit='Change',dtpref_cols='50',
     dtpref_rows='20',REQUEST=None): 
    """
    Added executeAt,periodDays to base DTMLMethod manage_edit
    """
    self._validateProxy(REQUEST)
    if self._size_changes.has_key(submit):
      return self._er(data,title,submit,dtpref_cols,dtpref_rows,REQUEST)
    self.title=title
    intExecuteAt_day=int(REQUEST.executeAt_day)
    day=REQUEST.executeAt_day
    if (REQUEST.executeAt_year%4==0) and (REQUEST.executeAt_month=='2') and (intExecuteAt_day>29):
      day='29'
    elif (REQUEST.executeAt_year%4!=0) and (REQUEST.executeAt_month=='2') and (intExecuteAt_day>28):
      day='28'
    elif (REQUEST.executeAt_month in ['4','6','9','11']) and (intExecuteAt_day>30):
      day='30'
    executeAt=DateTime(str(REQUEST.executeAt_year)+'/'+REQUEST.executeAt_month+'/'+day+' '+REQUEST.executeAt_hour+':'+str(REQUEST.executeAt_minute)+':'+str(REQUEST.executeAt_second))
    self.executeAt=executeAt
    self.originalDate=executeAt
    self.periodDays=REQUEST.periodDays
    self.periodDays=REQUEST.periodDays
    self.frequency=REQUEST.frequency
    self.reindex_object()
    if type(data) is not type(''): data=data.read()
    self.munge(data)
    if REQUEST:
      message="Content changed."
      return self.manage_main(self,REQUEST,manage_tabs_message=message)

default__class_init__(XronDTMLMethod)

manage_addXronDTMLMethodForm = HTMLFile('dtml/addXronDTMLMethod', globals())

def manage_addXronDTMLMethod(self,id,executeAt_day,executeAt_month,
    executeAt_year,executeAt_hour,executeAt_minute,executeAt_second,
    periodDays,frequency,title='',file='',REQUEST=None,submit=None):
  """Add an XronDTMLMethod object with the contents of file. 
  If 'file' is empty, default document text is used.
  """
  if type(file) is not type(''):
    file=file.read()
  if not file:
    file=default_event_html
  intExecuteAt_day=int(executeAt_day)
  if (executeAt_year%4==0) and (executeAt_month=='2') and (intExecuteAt_day>29):
    executeAt_day='29'
  elif (executeAt_year%4!=0) and (executeAt_month=='2') and (intExecuteAt_day>28):
    executeAt_day='28'
  elif (executeAt_month in ['4','6','9','11']) and (intExecuteAt_day>30):
    executeAt_day='30'
  executeAt=DateTime(str(executeAt_year)+'/'+executeAt_month+'/'+executeAt_day+' '+executeAt_hour+':'+str(executeAt_minute)+':'+str(executeAt_second))
  event=XronDTMLMethod(id,executeAt,periodDays,frequency,title,file)
  id=self._setObject(id, event)
  if REQUEST is not None:
    try: u=self.DestinationURL()
    except: u=REQUEST['URL1']
    if submit==" Add and Edit ": u="%s/%s" % (u,quote(id))
    REQUEST.RESPONSE.redirect(u+'/manage_main')
  return ''
